<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
if (!defined('CPO')) exit;

$content    = array();
$replacer   = array();
$iftags     = array();
$db_content = new DB;
$fetchedcontent = array(); // stores which content type was fetched already, to minimize database queries

if (!isset($placeholders_cache)) $placeholders_cache = array();

// fetch settings for all placeholders of the active template
$placeholder = array();
if (!empty($templates_id)) {
    if (!array_key_exists($templates_id, $placeholders_cache)) {
        $placeholders_cache[$templates_id] = array();

        $dbq = new DB;
        $dbq->query('SELECT ttype, fieldname, ddata '
                   .'FROM '.CPO_PLACEHOLDERS.' '
                   .'WHERE templates_id='.(int)$templates_id);
        while($dbq->next_record()) {
            $phname = $dbq->v('fieldname').' ('.$typeNames[$dbq->v('ttype')].')';
            $placeholders_cache[$templates_id][$phname] = unserialize($dbq->v('ddata'));
        }
    }

    $placeholder = $placeholders_cache[$templates_id];
}

// create a regular expression with all available placeholders
$tagsPreg = '';
$_div = '';
foreach ($hdlr_parser_replacement as $k=>$v) {
    $tagsPreg .= $_div.preg_quote($k, '/');
    $_div = '|';
}
$tagsPreg = '/<!--\s*('.$tagsPreg.')\s*:\s*([^;]*?)\s*(;\s*(.*?)\s*)?-->/si';

// find and process all tags
preg_match_all($tagsPreg, $template_datei_full, $matches, PREG_SET_ORDER);
foreach ($matches as $match) {
    $_fullmatch = $match[0];
    $_type      = strtolower(trim($match[1]));
    $_name      = preg_replace("/(\r\n|\n|\r)/", ' ', trim($match[2]));
    $key        = $_name.' ('.$_type.')';
    $_params    = '';
    if (isset($match[4])) $_params = preg_replace("/(\r\n|\n|\r)/", ' ', trim($match[4]));

    // fetch content for this placeholder if not already done so
    if (!isset($content[$key])
        && array_key_exists($_type, $hdlr_parser_content)) {

        doActions($hdlr_parser_content[$_type]);
    }

    // if we don't have content for this placeholder, replace it with an empty string
    // FIXME is this necessary? Should it be done by parser_r_*? How much performance do we get?
    if (!array_key_exists($key, $content) || $content[$key]==false) {
        $replacer['/'.preg_quote($_fullmatch, '/').'/'] = '';
        continue;
    }

    // create an array with all parameters
    $parameters = array();
    foreach (explode(';', $_params) as $value) {
        $value = trim($value);
        if ($value=='') continue;

        // split 'a = b' or only 'a'
        preg_match('/\s*([^=]*)\s*(=(.*))?/', $value, $tmp);

        // no matches found?
        if (count($tmp)==0) continue;

        if (count($tmp)==2) { // only a flag
            $parameters[strtolower(trim($tmp[1]))] = true;
        }
        if (count($tmp)==4) { // a = b
            $parameters[strtolower(trim($tmp[1]))] = trim($tmp[3]);
        }
    }

    // create a replacement for the tag
    $replacement = '<!-- ERROR: unable to find replacement for the following placeholder (unknown type?): -->'.$_fullmatch;
    if (array_key_exists($_type, $hdlr_parser_replacement)) {
        doActions($hdlr_parser_replacement[$_type]);
    }
    $replacer['/'.preg_quote($_fullmatch, '/').'/'] = $replacement;
}

// search for ifSet / ifNotSet tags and create a second replacer-array "iftags"
preg_match_all('/<!--\s*(ifset|ifnotset)\s*:\s*([^;]*?)\s*(;\s*(.*?)\s*)?-->/si', $template_datei, $matches, PREG_SET_ORDER);
foreach ($matches as $match) {
    $_fullmatch = $match[0];
    $_type      = strtolower(trim($match[1]));
    $_name      = preg_replace("/(\r\n|\n|\r)/", ' ', trim($match[2]));

    $key = $_name.' ('.$_type.')';

    $_regex = sprintf('/%s/', preg_quote($_fullmatch, '/'));
    switch ($_type) {
    case 'ifset':
        $found = false;
        foreach ($content as $key2=>$value2) {
            if ($value2['name']!=$_name) continue;
            if (substr($_regex, -31, 31)=='(.*?)\<\!--\s*\/ifSet\s*--\>/si') continue; 

            // we got content for this placeholder
            $_regex = substr($_regex, 0, -1).'(.*?)\<\!--\s*\/ifSet\s*--\>/si';
            $iftags[$_regex] = '\1';
            $found= true;
        }
        if ($found) break;

        // content not found
        $_regex = substr($_regex, 0, -1).'(.*?)\<\!--\s*\/ifSet\s*--\>/si';
        $iftags[$_regex] = '';
        break;

    case 'ifnotset': 
        $found = false;
        foreach ($content as $key2=>$value2) {
            if ($value2['name']!=$_name) continue;
            if (substr($_regex, -34, 34)=='(.*?)\<\!--\s*\/ifNotSet\s*--\>/si') continue;

            // we got content for this placeholder
            $_regex = substr($_regex, 0, -1).'(.*?)\<\!--\s*\/ifNotSet\s*--\>/si';
            $iftags[$_regex] = '';
            $found= true;
        }
        if ($found) break;

        // content not found
        $_regex = substr($_regex, 0, -1).'(.*?)\<\!--\s*\/ifNotSet\s*--\>/si';
        $iftags[$_regex] = '\1';
        break;

    default: 
        break;
    }
}

// do the magic pattern replacing 
if (count($iftags)>0)
    $template_datei = preg_replace(array_keys($iftags), array_values($iftags), $template_datei);

if (count($replacer)>0)
    $template_datei = preg_replace(array_keys($replacer), array_values($replacer), $template_datei);
?>
